﻿DeclareModule Share
UseModule AIMP

Declare Init(AIMP=#False)
Declare Free(AIMP=#False)
Declare SetCB(*CB, Flag.a)
Declare IsData()
Declare IsAimp()
Declare GetShare(*Data.TAIMPVisualData)
Declare SetShare(*Data.TAIMPVisualData)
EndDeclareModule


Module Share

EnableExplicit

#Mutex_TimeOut = 4000
#Event_TimeOut = 1000

Prototype pCB(*Buff)

Structure ShareData
  VisualData.TAIMPVisualData
  CountProcess.l ; Количество процессов, в которые загружена DLL.
  Counter.l      ; При при наличии новых данных записывается число из CountProcess. 
                 ; Когда процесс получит данные, необходимо уменишить значение на 1.
  IsAimp.a       ; Признак использования этой DLL плеером.
EndStructure

Structure Params
  hMutex.i
  hEvent.i
  ThreadID.i
  BreakThread.a
  *CB.pCB
  SendDataCB.a
  FlagNewData.a
  IsAimpProcess.a ; Признак использования этой DLL плеером.
EndStructure

Global *gShareData.ShareData, gParam.Params

Procedure Thread(*x)
  Protected VisualData.TAIMPVisualData
  
  With gParam
    
    Repeat
      Delay(10)
      
      If WaitForSingleObject_(\hEvent, #Event_TimeOut) = #WAIT_OBJECT_0
        
        If \BreakThread = #True
          Break
        EndIf
        
        If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
          
          \FlagNewData = #True
        
          *gShareData\Counter - 1
          If *gShareData\Counter <= 0
            ResetEvent_(\hEvent)
          EndIf
          
          If \CB And \SendDataCB = #True
            CopyStructure(@*gShareData\VisualData, @VisualData, TAIMPVisualData)
          EndIf
          
          ReleaseMutex_(\hMutex)
        EndIf
        
        If \CB
          If \SendDataCB = #True
            \CB(@VisualData)
          Else
            \CB(0)
          EndIf
        EndIf
        
      EndIf
      
    Until \BreakThread = #True
    \BreakThread+1
  EndWith
EndProcedure

Procedure SetCB(*CB, Flag.a)
  With gParam
    \CB = *CB
    \SendDataCB = Flag
  EndWith
EndProcedure

Procedure IsData()
  ProcedureReturn gParam\FlagNewData
EndProcedure

Procedure IsAimp()
  Protected Res = #False
  
   With gParam
    If \hMutex
      If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        If *gShareData\IsAimp = #True
          Res = #True
        EndIf
        ReleaseMutex_(\hMutex)
      EndIf
    EndIf
  EndWith
  
  ProcedureReturn Res
EndProcedure

Procedure GetShare(*Data.TAIMPVisualData)
  Protected Res = #False
  
  With gParam
    If \hMutex And *Data
      If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        CopyMemory(@*gShareData\VisualData, *Data, SizeOf(TAIMPVisualData))
        \FlagNewData = #False
        ReleaseMutex_(\hMutex)
        Res = #True
      EndIf
    EndIf
  EndWith
  
  ProcedureReturn Res
EndProcedure


Procedure SetShare(*Data.TAIMPVisualData)
  Protected Event = #False
  
  With gParam
    If \hMutex And \hEvent And *Data
      
      If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        If *gShareData\CountProcess > 0 ; Если DLL загружена в другие процессы кроме плеера.
          *gShareData\Counter = *gShareData\CountProcess
          CopyMemory(*Data, @*gShareData\VisualData, SizeOf(TAIMPVisualData))
          Event = #True
        EndIf
        ReleaseMutex_(\hMutex)
      EndIf
      
      If Event = #True
        SetEvent_(\hEvent)
      EndIf
      
    EndIf
  EndWith
EndProcedure


Procedure Init(AIMP=#False)
  Protected Str.s
  
  Free(AIMP)
  
  *gShareData = ?ShareData
  
  CompilerIf #PB_Compiler_Version >= 540
    UseMD5Fingerprint()
    Str = StringFingerprint(ProgramFilename(), #PB_Cipher_MD5)
  CompilerElse
    Str = ProgramFilename()
    Str = MD5Fingerprint(@Str, StringByteLength(Str))
  CompilerEndIf
  
  With gParam
    \CB = 0
    \IsAimpProcess = #False
    \hMutex = CreateMutex_(0, 0, "m"+Str)
    If \hMutex
      
      \hEvent = CreateEvent_(0, #True, #False, "e"+Str)
      
      \BreakThread = #False
      
      If AIMP=#False
        \ThreadID = CreateThread(@Thread(), 0)
        
        If \ThreadID
          If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
            *gShareData\CountProcess + 1
            ReleaseMutex_(\hMutex)
          EndIf
        EndIf
        
      Else
        *gShareData\IsAimp = #True ; Этот экземпляр dll загружен в плеер.
        \ThreadID = 0
        \IsAimpProcess = #True
      EndIf
      
    EndIf
  EndWith
EndProcedure

Procedure Free(AIMP=#False)
  Protected i
  
  With gParam
    
    If \ThreadID
      
      If IsThread(\ThreadID)
        
        \BreakThread = #True
        If \hEvent
          SetEvent_(\hEvent)        ; Чтобы быстрее завершился поток.
        EndIf
        
        For i=0 To #Event_TimeOut
          If \BreakThread = #True+1
            Break
          Else
            Delay(2)
          EndIf
        Next i
        
        If \BreakThread <> #True+1 And IsThread(\ThreadID)
          KillThread(\ThreadID)
        EndIf
      EndIf
      
      If \hMutex And WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        *gShareData\CountProcess - 1
        ReleaseMutex_(\hMutex)
      EndIf
      
      \ThreadID = 0
    EndIf
    
    
    If \hMutex
      If \IsAimpProcess = #True
        If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
          *gShareData\IsAimp = #False
          \IsAimpProcess = #False
          ReleaseMutex_(\hMutex)
        EndIf
      EndIf
      
      CloseHandle_(\hMutex)
      \hMutex = 0
    EndIf
    
    If \hEvent
      CloseHandle_(\hEvent)
      \hEvent=0
    EndIf
    
  EndWith
EndProcedure
  
DataSection
  ! section 'data' data readable writeable shareable
  ShareData:
  ! db 8192 dup (0)
  ! section '.data' data readable writeable
EndDataSection

EndModule
; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 161
; FirstLine = 154
; Folding = --
; EnableUnicode
; EnableXP